/*	config.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#define MAIN_OR_GLOBALS
#include "inc/freeEMS.h"


const fixedConfig configs FIXEDCONFIG = {
	/* Each bit represents the state of some core setting, masks below and above where the same one is used */
	/* Settings variables : 0 = false */
	/* unsigned short coreSettingsA = */	0x07F0,			/* See definitions in freeems.h */

	/* Pre configured value settings for the sensor inputs */
	/* unsigned short presetIAT = */		roomTemperature,
	/* unsigned short presetCHT = */		runningTemperature,
	/* unsigned short presetTPS = */		halfThrottle,
	/* unsigned short presetEGO = */		stoichiometricLambda,
	/* unsigned short presetBRV = */		runningVoltage,
	/* unsigned short presetMAP = */		idleManifoldPressure,
	/* unsigned short presetAAP = */		seaLevelKPa,
	/* unsigned short presetMAT = */		roomTemperature,
	/* unsigned short presetEGO2 = */		stoichiometricLambda,
	/* unsigned short presetIAP = */		maxExpectedBoost,
	/* unsigned short presetBPW = */		idlePulseWidth,
	/* unsigned short presetAF = */			idleAirFlow,


	/* Fuel injection settings */
	/* unsigned short perCylinderVolume = */	typicalCylinderSize,
	/* unsigned short stoichiometricAFR = */	stoichiometricOctane,
	/* unsigned short injectorFlow = */			typicalInjectorSize,
	/* unsigned short densityOfFuelAtSTP = */	densityOfOctane,


	/* Sensor related settings */
	/* unsigned short TPSClosedMAP = */		offIdleMAP,		/* Around 30kPa */
	/* unsigned short TPSOpenMAP = */		nearlyWOTMAP,	/* Around 95kPa */
	
	
	/* Sensor input conditioning settings */
	/* These are used to calculate MAP, EGO and TPS from ADC readings. */
	/* unsigned short MAPMinimum = */		MPX4250AMin,			/* Default to MPX4250A 260kPa - 8kPa = 252kPa See the link for the transfer function */
	/* unsigned short MAPRange = */			MPX4250ARange,			/* www.freescale.com/files/sensors/doc/data_sheet/MPX4250A.pdf */
	/* unsigned short AAPMinimum = */		MPX4100AMin,			/* Default to MPX4100A 107.5kPa - 14kPa = 93.5kPa See the link for the transfer function */
	/* unsigned short AAPRange = */			MPX4100ARange,			/* www.freescale.com/files/sensors/doc/data_sheet/MPX4100A.pdf */
	/* unsigned short EGOMinimum = */		LC1LambdaMin,			/* Default to Innovate LC-1 on lambda 0.5 - 1.5 for 0-5V range */
	/* unsigned short EGORange = */			LC1LambdaRange,			/* Range of lambda 1.0 */
	/* unsigned short BRVMinimum = */		batteryVoltageMin,		/* 0 - 24.5 Volt measurement with 10k and 39k resistors */
	/* unsigned short BRVRange = */			batteryVoltageRange,	/* http://www.google.com/search?hl=en&safe=off&q=5+*+(39000+%2B+10000)+%2F+10000&btnG=Search */
	/* unsigned short TPSMinimumADC = */	TPSDefaultMin,			/* 25% of voltage = closed (0%) */
	/* unsigned short TPSMaximumADC = */	TPSDefaultMax,			/* 75% of voltage = open (100%) */


	/* Setting variables (must be inited with useful values) */
	/* unsigned char ports = */									6,		/* Default to maximum */
	/* unsigned char coils = */									6,		/* Default to  */
	/* unsigned char combustionEventsPerEngineCycle = */		6,		/* Default to  */
	/* unsigned char revolutionsPerEngineCycle = */				2,		/* Default to  */
	/* unsigned short capacityOfAirPerCombustionEvent = */		500,	/* Default to TODO duplicated above */
	/* unsigned short perPrimaryInjectorChannelFlowRate = */	550,	/* Default to  */
	/* unsigned short perSecondaryInjectorChannelFlowRate = */	550,	/* Default to  */
	/* unsigned short readingTimeout = */						500,	/* Default to half a second 60rpm for a 4 cylinder */
	/* unsigned char primaryTeeth = */							24,		/* Default to  */
	/* unsigned char missingTeeth = */							0,		/* Default to  */
	
	/* temporary contents for notes etc */
	/* unsigned char userTextField[] = */	{"Place your personal notes here!"}
};
