/*	freeEMS.h

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

/* Header file multiple inclusion protection courtesy eclipse Header Template	*/
/* and http://gcc.gnu.org/onlinedocs/gcc-3.1.1/cpp/ C pre processor manual		*/
#ifndef FILE_FREEEMS_H_SEEN
#define FILE_FREEEMS_H_SEEN


/* Include top level files that everything else relies on */
#include "memory.h"
#include "9S12XDP512.h"


#ifdef MAIN_OR_GLOBALS
#define EXTERN
#else
#define EXTERN extern
#endif


/* Include define files at the top here as other includes use them */
#include "errorDefines.h"
#include "globalDefines.h"


/* Include items that would normally be found in here */
#include "structs.h"
#include "globalConstants.h"


/* Types summary
 * 
 * BEWARE : Be explicit!!
 * 					
 * char			8 bit (defaults to unsigned, but always specify signed/unsigned anyway)
 * short		16 bit (defaults to signed, but always specify signed/unsigned anyway)
 * int			16 bit DO NOT USE! (current compile flags make this 16 bits, but a change of flags could will change your program if you use this because they will all be 32 bit all of a sudden)
 * long			32 bit (defaults to signed, but always specify signed/unsigned anyway)
 * long long	64 bit (inefficient, avoid these, if using : defaults to signed, but always specify signed/unsigned anyway)
 * float		32 bit IEEE floating point numbers (inefficient, avoid these, used fixed point math)
 * double		64 bit IEEE floating point numbers (inefficient, avoid these, used fixed point math)
 */


/* GLOBAL Variables */
// TODO change any of these that need it to volatile!!!


// temporary test vars
EXTERN unsigned char portHDebounce; // init to zero required
EXTERN unsigned short RPM[IN_OUT_BANKS]; // to be replaced with logging scheme for teeth.


/* Declare instances of variable structs for use */
EXTERN Clock Clocks;				/* Timer Clocks for various functions */
EXTERN Counter Counters;			/* Execution count for various blocks of code */
EXTERN RuntimeVar RuntimeVars;	/* Execution times for various blocks of code */

EXTERN CoreVar CoreVars;			/* main running variables */
EXTERN DerivedVar DerivedVars;		/* secondary running variables */

EXTERN ADCArray ADCArrays;		/* main adc storage area for syncronous sampling in the engine position ISR or injection ISR or ignition ISR etc. */
EXTERN ADCArray asyncADCArrays;	/* secondary adc storage area for asynchronously sampling in the RTC/RTI ISR */


/* Potentially pointers for data in ram depending on how it gets implemented */
//EXTERN config *configp;
//EXTERN mainTable *veTableMain;
//EXTERN mainTable *veTableSec;
//EXTERN mainTable *advTableMain;
//EXTERN mainTable *advTableSec;


/* Output variables (init not required) TODO ditch this in favour of the real vars in the calcs function and struct */
EXTERN unsigned short masterPulseWidth;
EXTERN unsigned short totalDwell;
EXTERN unsigned short totalAngleAfterReferenceInjection;
EXTERN unsigned short totalAngleAfterReferenceIgnition;

EXTERN unsigned long bootFuelConst;	/* constant derived from configurable constants */
EXTERN unsigned short TPSMAPRange;	/* The MAP range used to convert fake TPS from MAP and vice versa */
EXTERN unsigned short TPSADCRange;	/* The ADC range used to generate TPS percentage */
EXTERN unsigned short boundedTPSADC; // temp to view to debug

EXTERN unsigned short bootTimeAAP; /* TODO populate this at switch on time depending on a few things. */

/* ALL STATUS STUFF HERE */

/* State variables : 0 = false (don't forget to change the init mask to suit!) */
EXTERN unsigned short coreStatusA;	/* Each bit represents the state of some core parameter, masks below */
/* Bit masks for coreStatusA */ // TODO needs a rename as does coresetingsA
#define RPM_VALID		BIT1_16		/*  1 Whether we are sure rpm is what the variable says (used to inject fuel without ignition below the threshold rpm) */
#define PRIMARY_SYNC	BIT2_16		/*  2 Wasted spark/Semi sequential */
#define SECONDARY_SYNC	BIT3_16		/*  3 COP/Full sequential */
#define ENGINE_PHASE	BIT4_16		/*  4 For COP/Sequential, which revolution we are in, first or second */
#define FUEL_CUT		BIT5_16		/*  5 Remove injection completely */
#define HARD_SPARK_CUT	BIT6_16		/*  6 Remove ignition completely */
#define SOFT_SPARK_CUT	BIT7_16		/*  7 Remove ignition events round robin style */
#define SPARK_RETARD	BIT8_16		/*  8 Retard ignition in RPM dependent way */
#define STAGED_REQUIRED	BIT9_16		/*  9 Whether we are firing the staged injectors */
#define CALC_FUEL_IGN	BIT10_16	/* 10 Whether fuel and ignition require calculation (i.e. variables have been updated) */
#define FORCE_READING	BIT11_16	/* 11 Flag to force ADC sampling at low rpm/stall */
#define CALCS_VALID		BIT12_16	/* 12 Whether the calculated outputs are valid or not */
#define COREA13			BIT13_16	/* 13 */
#define COREA14			BIT14_16	/* 14 */
#define COREA15			BIT15_16	/* 15 */
#define COREA16			BIT16_16	/* 16 */


/* Which bank to read from or write to at any time
 *
 * This banking system has four regions and four main controlling variables.
 * Each pair of variables should always be opposite each other, correct values being zero and one.
 * The two pairs can be reversed independently and indeed, this is how it functions.
 * Additionally, there are two pairs of offset variables for the math output/real time side
 * that depend on the value of the second main pair and should be consistent with those.
 */

/* These two should always be opposite to each other */

EXTERN unsigned char recordADCBank;		/* For reading ADC inputs into */
EXTERN unsigned char mathInputBank;		/* For reading values for math */

/* This pair of tripplets should always be opposite to each other */

/* For writing values out from math and reuse of values written out */
EXTERN unsigned char mathInternalBank;
EXTERN unsigned char mathInternalInjectionOffset;
EXTERN unsigned char mathInternalIgnitionOffset;

/* For reading values out of */
EXTERN unsigned char realTimeUseBank;
EXTERN unsigned char realTimeUseInjectionOffset;
EXTERN unsigned char realTimeUseIgnitionOffset;


//TODO make this volatile?
/* ECT IC extension variable (init not required, don't care where it is, only differences between figures) */
unsigned short timerExtensionClock;						/* Increment for each overflow of the main timer, allows finer resolution and longer time period */
/* section 10.3.5 page 290 68hc11 reference manual e.g. groups.csail.mit.edu/drl/courses/cs54-2001s/pdf/M68HC11RM.pdf */


/* For extracting 32 bit long time stamps from the overflow counter and timer registers */
typedef union { /* Declare Union http://www.esacademy.com/faq/docs/cpointers/structures.htm */
    unsigned long timeLong;
    unsigned short timeShorts[2];
} LongTime;


/* Flag registers, init to zero required */
EXTERN unsigned char mainOn;				/* Keep track of where we are at for possible use as multi interrupt per injection */
EXTERN unsigned short dwellOn;				/* Keep track of ignition output state */
EXTERN unsigned char stagedOn;				/* Ensure we turn an injector off again if we turn it on. */
EXTERN unsigned char selfSetTimer;			/* Set the start time of injection at the end of the last one in the channels ISR instead of the input ISR */
EXTERN unsigned char rescheduleFuelFlags;	/* Pulse width is probably longer than engine cycle so schedule a restart at the next start time */


/* Engine Position and RPM reading variables */

/* Engine runtime properties (inits???) TODO */
EXTERN unsigned short primaryPulsesPerSecondaryPulse; /* Type short because of nissan style cam wheels (char would do for other types) */
EXTERN unsigned short primaryPulsesPerSecondaryPulseBuffer; /* Type short because of nissan style cam wheels (char would do for other types) */
EXTERN unsigned short primaryLeadingEdgeTimeStamp; /* Store the timestamp of the leading edge during a pulse */
EXTERN unsigned short primaryTrailingEdgeTimeStamp; /* Store the timestamp of the leading edge during a pulse */
EXTERN unsigned short timeBetweenSuccessivePrimaryPulses; /* This number equates to the speed of the engine */
EXTERN unsigned short timeBetweenSuccessivePrimaryPulsesBuffer; /* This number equates to the speed of the engine */
EXTERN unsigned short lastPrimaryPulseTimeStamp; /* Store the timer value of the each pulse here before exiting the ISR */
EXTERN unsigned long engineCyclePeriod; /* Timer units between engine cycle starts */
EXTERN unsigned long lastSecondaryOddTimeStamp;


/* Ignition stuff */

// ignition experimentation stuff
EXTERN unsigned char dwellQueueLength;				/* 0 = no dwell pending start, 1 = single event scheduled, 2 = one scheduled, and one in the queue, etc */
EXTERN unsigned char ignitionQueueLength;			/* 0 = no spark event pending, 1 = single event scheduled, 2 = one scheduled, and one in the queue, etc */
EXTERN unsigned short currentDwell[IN_OUT_BANKS];	/* Current dwell to be used. */
EXTERN unsigned char nextDwellChannel;				/* Which one to bang off next */
EXTERN unsigned char nextIgnitionChannel;			/* Which one to bang off next */
EXTERN unsigned short ignitionAdvances[IGNITION_CHANNELS * 2]; // Uses channel + offset to have two values at any time
EXTERN unsigned short queuedDwellOffsets[IGNITION_CHANNELS]; // Uses next channel as index
EXTERN unsigned short queuedIgnitionOffsets[IGNITION_CHANNELS]; // Uses next channel as index


/* Injection stuff */

/* Register addresses */
EXTERN volatile unsigned short * volatile injectorMainTimeRegisters[INJECTION_CHANNELS];
EXTERN volatile unsigned char * volatile injectorMainControlRegisters[INJECTION_CHANNELS];  

/* Timer holding vars (init not required) */
EXTERN unsigned short injectorMainStartTimesHolding[INJECTION_CHANNELS];
EXTERN unsigned long injectorMainEndTimes[INJECTION_CHANNELS];

// TODO make these names consistent
/* Code time to run variables (init not required) */
EXTERN unsigned short injectorCodeOpenRuntimes[INJECTION_CHANNELS];
EXTERN unsigned short injectorCodeCloseRuntimes[INJECTION_CHANNELS];

/* individual channel pulsewidths (init not required) */
EXTERN unsigned short injectorMainPulseWidths[INJECTION_CHANNELS * 2];
EXTERN unsigned short injectorStagedPulseWidths[INJECTION_CHANNELS * 2];

/* Channel latencies (init not required) */
EXTERN unsigned short injectorCodeLatencies[INJECTION_CHANNELS];

#undef IN_OUT_BANKS
#undef EXTERN

#else
	/* let us know if we are being untidy with headers */
	#warning "Header file FREEEMS_H seen before, sort it out!"
/* end of the wrapper ifdef from the very top */
#endif
