/*	memory.h

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

/* Header file multiple inclusion protection courtesy eclipse Header Template	*/
/* and http://gcc.gnu.org/onlinedocs/gcc-3.1.1/cpp/ C pre processor manual		*/
#ifndef FILE_MEMORY_H_SEEN
#define FILE_MEMORY_H_SEEN

/* http://gcc.gnu.org/onlinedocs/gcc-4.0.0/gcc/Variable-Attributes.html	*/

// EEPROM
#define EEPROM __attribute__ ((section (".eeprom")))

// Direct flash blocks TODO divide these up into suitable blocks for straight copying to ram
#define FIXEDCONFIG __attribute__ ((section (".text1")))
#define TUNABLECONFIG __attribute__ ((section (".text2"))) /* temporary use till hcs12mem fixed */
/* #define TEXT __attribute__ ((section (".text"))) Not required, code defaults to being stored there. */

/* far shortcut for data */
#define DFAR(label) __attribute__ ((section (label)))

// Paged flash blocks for data
#define PPAGE_E0 DFAR(".ppageE0") /* temporarily remove this till hcs12mem fixed */ 
#define PPAGE_E1 DFAR(".ppageE1")
#define PPAGE_E2 DFAR(".ppageE2")
#define PPAGE_E3 DFAR(".ppageE3")
#define PPAGE_E4 DFAR(".ppageE4")
#define PPAGE_E5 DFAR(".ppageE5")
#define PPAGE_E6 DFAR(".ppageE6")
#define PPAGE_E7 DFAR(".ppageE7")
#define PPAGE_E8 DFAR(".ppageE8")
#define PPAGE_E9 DFAR(".ppageE9")
#define PPAGE_EA DFAR(".ppageEA")
#define PPAGE_EB DFAR(".ppageEB")
#define PPAGE_EC DFAR(".ppageEC")
#define PPAGE_ED DFAR(".ppageED")
#define PPAGE_EE DFAR(".ppageEE")
#define PPAGE_EF DFAR(".ppageEF")
#define PPAGE_F0 DFAR(".ppageF0")
#define PPAGE_F1 DFAR(".ppageF1")
#define PPAGE_F2 DFAR(".ppageF2")
#define PPAGE_F3 DFAR(".ppageF3")
#define PPAGE_F4 DFAR(".ppageF4")
#define PPAGE_F5 DFAR(".ppageF5")
#define PPAGE_F6 DFAR(".ppageF6")
#define PPAGE_F7 DFAR(".ppageF7")
#define PPAGE_F8 DFAR(".ppageF8")
#define PPAGE_F9 DFAR(".ppageF9")
#define PPAGE_FA DFAR(".ppageFA")
#define PPAGE_FB DFAR(".ppageFB")
#define PPAGE_FC DFAR(".ppageFC")
#define PPAGE_FD DFAR(".ppageFD") /* Represents 0x4000 - 0x7FFF (TEXT1 paged) ????????? */
#define PPAGE_FE DFAR(".ppageFE") /* whats this then?? text2? */
#define PPAGE_FF DFAR(".ppageFF") /* Represents 0xC000 - 0xFFFF (TEXT paged) ?????????? */
/* The previous two lines are included for clarity only. */
/* Changes to the memory layout should be reflected in memory.x and Makefile also */

/* far shortcut for functions */
#define FAR(label) __attribute__ ((far)) __attribute__ ((section (label)))

// Paged flash blocks for functions
#define FPPAGE_E0 FAR(".ppageE0") /* temporarily remove this till hcs12mem fixed */
#define FPPAGE_E1 FAR(".ppageE1")
#define FPPAGE_E2 FAR(".ppageE2")
#define FPPAGE_E3 FAR(".ppageE3")
#define FPPAGE_E4 FAR(".ppageE4")
#define FPPAGE_E5 FAR(".ppageE5")
#define FPPAGE_E6 FAR(".ppageE6")
#define FPPAGE_E7 FAR(".ppageE7")
#define FPPAGE_E8 FAR(".ppageE8")
#define FPPAGE_E9 FAR(".ppageE9")
#define FPPAGE_EA FAR(".ppageEA")
#define FPPAGE_EB FAR(".ppageEB")
#define FPPAGE_EC FAR(".ppageEC")
#define FPPAGE_ED FAR(".ppageED")
#define FPPAGE_EE FAR(".ppageEE")
#define FPPAGE_EF FAR(".ppageEF")
#define FPPAGE_F0 FAR(".ppageF0")
#define FPPAGE_F1 FAR(".ppageF1")
#define FPPAGE_F2 FAR(".ppageF2")
#define FPPAGE_F3 FAR(".ppageF3")
#define FPPAGE_F4 FAR(".ppageF4")
#define FPPAGE_F5 FAR(".ppageF5")
#define FPPAGE_F6 FAR(".ppageF6")
#define FPPAGE_F7 FAR(".ppageF7")
#define FPPAGE_F8 FAR(".ppageF8")
#define FPPAGE_F9 FAR(".ppageF9")
#define FPPAGE_FA FAR(".ppageFA")
#define FPPAGE_FB FAR(".ppageFB")
#define FPPAGE_FC FAR(".ppageFC")
#define FPPAGE_FD FAR(".ppageFD") /* Represents 0x4000 - 0x7FFF (TEXT1 paged) ??????????? */
#define FPPAGE_FE FAR(".ppageFE") /* whats this then?? text2 ?*/
#define FPPAGE_FF FAR(".ppageFF") /* Represents 0xC000 - 0xFFFF (TEXT paged) ??????????? */
/* The previous two lines are included for clarity only. */
/* Changes to the memory layout should be reflected in memory.x and Makefile also */

#else
	/* let us know if we are being untidy with headers */
	#warning "Header file MEMORY_H seen before, sort it out!"
/* end of the wrapper ifdef from the very top */
#endif
